package uy.edu.fing.chesstrack.modulomodelador;

import org.opencv.core.Core;
import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;
import org.opencv.core.Size;
import org.opencv.imgproc.Imgproc;

import uy.edu.fing.chesstrack.modulovision.Calibracion;
import android.util.Log;

public class Modelador {
	private static final String TAG = "CHESSTRACK::Modelador";
	private final int CANT_FILAS = 8;
	private final int CANT_SCAQUE = CANT_FILAS * CANT_FILAS;
	private final int MIN_CANT_PIX_WHITE = 50;
	private Mat tablero;
	// se cuentan de arrib a abajo y de izquierda a derecha
	private final Mat[][] escaques;

	// private static Modelador instance;

	public Modelador() {
		tablero = Mat.zeros(new Size(8, 8), CvType.CV_8UC1);
		escaques = new Mat[CANT_FILAS][CANT_FILAS];

	}

	public void dividirTablero(Mat inputFrame) {
		Imgproc.cvtColor(inputFrame, inputFrame, Imgproc.COLOR_RGBA2GRAY, 4);
		int largo = inputFrame.rows();
		int ancho = inputFrame.cols();
		// obtengo cuanto mide en ancho y largo un escaque
		int largoEscaque = (int) Math.floor(largo / (CANT_FILAS + 2));// +2 porque se toma un escaque mas de borde
		int anchoEscaque = (int) Math.floor(ancho / (CANT_FILAS + 2));
		Log.i(TAG, "largoEscaque= " + largoEscaque);
		Log.i(TAG, "anchoEscaque= " + anchoEscaque);
		for (int i = 0; i < CANT_FILAS; i++) {
			for (int j = 0; j < CANT_FILAS; j++) {
				int rowStart = (int) Math.floor(largoEscaque) + i
						* largoEscaque + 4;
				int rowEnd = (int) Math.floor(largoEscaque) + i * largoEscaque
						+ largoEscaque - 4;
				int colStart = (int) Math.floor(anchoEscaque) + j
						* anchoEscaque + 4;
				int colEnd = (int) Math.floor(anchoEscaque) + j * anchoEscaque
						+ anchoEscaque - 4;
				escaques[i][j] = inputFrame.submat(colStart, colEnd, rowStart,
						rowEnd);
			}
		}
	}

	private int getHayFichaEnEscaque(int i, int j) {
		int ret = 0;
		Mat mIntermediateMat = new Mat();
		Imgproc.Sobel(escaques[i][j], mIntermediateMat, CvType.CV_8U, 1, 1);
		Core.convertScaleAbs(mIntermediateMat, mIntermediateMat, 10, 0);
		Imgproc.threshold(mIntermediateMat, mIntermediateMat, 70, 255,
				Imgproc.THRESH_BINARY);
		int cant_pix = mIntermediateMat.cols() * mIntermediateMat.rows();
		int cant_pix_white = Core.countNonZero(mIntermediateMat);
		if (cant_pix_white > (MIN_CANT_PIX_WHITE)) {
			ret = 1;
		}
		return ret;
	}

	public Mat getMatrizFichas(Mat frame) {
		dividirTablero(frame);
		tablero = Mat.zeros(new Size(8, 8), CvType.CV_8UC1);
		for (int i = 0; i < CANT_FILAS; i++) {
			for (int j = 0; j < CANT_FILAS; j++) {
				tablero.put(i, j, getHayFichaEnEscaque(i, j));
			}
		}
		return tablero;
	}

	public Mat dibujarEscaque() {
		Mat tmp = Mat.ones(Calibracion.getInstance().get_sizeFrame(),
				CvType.CV_8UC4);
		Mat subm; // tmp.submat(new Rect(new Point(0,0)
		// ,escaques[i][j].size()));
		// escaques[i][j].copyTo(subm);
		int pos_x = 0;
		int pos_y = 0;
		for (int i = 0; i < CANT_FILAS; i++) {
			for (int j = 0; j < CANT_FILAS; j++) {
				pos_x = (int) (escaques[i][j].size().width * i);
				pos_y = (int) (escaques[i][j].size().height * j);
				subm = tmp.submat(new Rect(new Point(pos_x, pos_y),
						escaques[i][j].size()));
				escaques[i][j].copyTo(subm);
				Core.putText(tmp,
						Integer.toString(i) + "-" + Integer.toString(j),
						new Point(pos_x + 1, pos_y + 1),
						Core.FONT_HERSHEY_SIMPLEX, 0.3, new Scalar(255, 0, 0));
			}
		}
		return tmp;
	}

	public Mat dibujarEscaquesCanny() {
		Mat tmp = Mat.zeros(Calibracion.getInstance().get_sizeFrame(),
				CvType.CV_8UC4);
		Mat subm; // tmp.submat(new Rect(new Point(0,0)
		// ,escaques[i][j].size()));
		// escaques[i][j].copyTo(subm);
		Mat mIntermediateMat = new Mat();
		int pos_x = 0;
		int pos_y = 0;
		for (int i = 0; i < CANT_FILAS; i++) {
			for (int j = 0; j < CANT_FILAS; j++) {
				pos_x = (int) (escaques[i][j].size().width * i);
				pos_y = (int) (escaques[i][j].size().height * j);
				subm = tmp.submat(new Rect(new Point(pos_x, pos_y),
						escaques[i][j].size()));

				Imgproc.Canny(escaques[i][j], mIntermediateMat, 80, 90);
				Imgproc.cvtColor(mIntermediateMat, subm,
						Imgproc.COLOR_GRAY2RGBA, 4);

				escaques[i][j].copyTo(subm);
				Core.putText(tmp, Integer.toString(i) + "-" + Integer.toString(j), new Point(pos_x + 1, pos_y + 1), Core.FONT_HERSHEY_SIMPLEX, 0.3, new Scalar(255, 0, 0));
			}
		}
		return tmp;
	}

	public Mat dibujarEscaquesSobel(int version) {
		Mat tmp = Mat.zeros(Calibracion.getInstance().getRectROI().size(),
				CvType.CV_8UC4);
		Mat subm; // tmp.submat(new Rect(new Point(0,0)
		// ,escaques[i][j].size()));
		// escaques[i][j].copyTo(subm);
		Mat mIntermediateMat = new Mat();
		int pos_x = 0;
		int pos_y = 0;
		for (int i = 0; i < CANT_FILAS; i++) {
			for (int j = 0; j < CANT_FILAS; j++) {
				pos_x = (int) (escaques[i][j].size().width * i);
				pos_y = (int) (escaques[i][j].size().height * j);
				subm = tmp.submat(new Rect(new Point(pos_x, pos_y),
						escaques[i][j].size()));
				// Imgproc.cvtColor(escaques[i][j] ,mIntermediateMat,
				// Imgproc.COLOR_RGBA2GRAY,4);
				Imgproc.Sobel(escaques[i][j], mIntermediateMat, CvType.CV_8U,
						1, 1);
				Core.convertScaleAbs(mIntermediateMat, mIntermediateMat, 10, 0);
				Imgproc.threshold(mIntermediateMat, mIntermediateMat, 70, 255,
						Imgproc.THRESH_BINARY);
				int cant_pix = mIntermediateMat.cols()
						* mIntermediateMat.rows();
				int cant_pix_white = Core.countNonZero(mIntermediateMat);
				Log.i(TAG, "cant_pix=" + cant_pix);
				Log.i(TAG, "cant_pix_white=" + cant_pix_white);
				switch (version) {
				case 1:
					if (cant_pix_white > (MIN_CANT_PIX_WHITE)) {
						Imgproc.cvtColor(mIntermediateMat, subm, Imgproc.COLOR_GRAY2RGBA, 4);
					} else {
						Imgproc.cvtColor(escaques[i][j], subm, Imgproc.COLOR_GRAY2RGBA, 4);
					}
					break;
				case 2:
					Imgproc.cvtColor(mIntermediateMat, subm, Imgproc.COLOR_GRAY2RGBA, 4);
					break;
				default:
					Imgproc.cvtColor(escaques[i][j], subm, Imgproc.COLOR_GRAY2RGBA, 4);
					break;

				}
			}
		}
		return tmp;
	}

}
